require 'digest/sha1'

# this model expects a certain database layout and its based on the name/login pattern. 
class User < ActiveRecord::Base
	has_many :projects
	has_many :lines, :foreign_key=>'sender_id'
	
#These methods are for the chat-user interactions 	
 def say_in_channel(channel, say)
    Message.new(:channel => channel, :content => sanitize(say), :sender => self).save
  end

  def action_in_channel(channel, say)
    Message.new(:channel => channel, :content => sanitize(say), :sender => self, :level => 'action').save
  end
  
  def joins_channel(channel)
    Message.new(:channel => channel, :sender => self, :content => "joins the channel", :level => 'sys').save
  end
  
  def leaves_channel(channel)
    Message.new(:channel => channel, :sender => self, :content => "left the channel", :level => 'sys').save
  end

  def sanitize(say)
    say.gsub!(/\</, '&lt;')
    say.gsub!(/\>/, '&gt;')
    say
  end

	
	
	
#These methods are for the user-login interactions	
  def self.authenticate(login, pass)
    find_first(["login = ? AND password = ?", login, sha1(pass)])
  end  

  def change_password(pass)
    update_attribute "password", self.class.sha1(pass)
  end
    
  protected

  def self.sha1(pass)
    Digest::SHA1.hexdigest("change-me--#{pass}--")
  end
    
  before_create :crypt_password
  
  def crypt_password
    write_attribute("password", self.class.sha1(password))
  end
  
   
  validates_length_of :login, :within => 3..40
  validates_length_of :password, :within => 5..40
  validates_presence_of :login, :password, :password_confirmation
  validates_uniqueness_of :login, :on => :create
  validates_confirmation_of :password, :on => :create     
end
