require File.dirname(__FILE__) + '/../test_helper'
require 'paragraphs_controller'

# Re-raise errors caught by the controller.
class ParagraphsController; def rescue_action(e) raise e end; end

class ParagraphsControllerTest < Test::Unit::TestCase
  fixtures :paragraphs

	NEW_PARAGRAPH = {}	# e.g. {:name => 'Test Paragraph', :description => 'Dummy'}
	REDIRECT_TO_MAIN = {:action => 'list'} # put hash or string redirection that you normally expect

	def setup
		@controller = ParagraphsController.new
		@request    = ActionController::TestRequest.new
		@response   = ActionController::TestResponse.new
		# Retrieve fixtures via their name
		# @first = paragraphs(:first)
		@first = Paragraph.find_first
	end

  def test_component
    get :component
    assert_response :success
    assert_template 'paragraphs/component'
    paragraphs = check_attrs(%w(paragraphs))
    assert_equal Paragraph.find(:all).length, paragraphs.length, "Incorrect number of paragraphs shown"
  end

  def test_component_update
    get :component_update
    assert_response :redirect
    assert_redirected_to REDIRECT_TO_MAIN
  end

  def test_component_update_xhr
    xhr :get, :component_update
    assert_response :success
    assert_template 'paragraphs/component'
    paragraphs = check_attrs(%w(paragraphs))
    assert_equal Paragraph.find(:all).length, paragraphs.length, "Incorrect number of paragraphs shown"
  end

  def test_create
  	paragraph_count = Paragraph.find(:all).length
    post :create, {:paragraph => NEW_PARAGRAPH}
    paragraph, successful = check_attrs(%w(paragraph successful))
    assert successful, "Should be successful"
    assert_response :redirect
    assert_redirected_to REDIRECT_TO_MAIN
    assert_equal paragraph_count + 1, Paragraph.find(:all).length, "Expected an additional Paragraph"
  end

  def test_create_xhr
  	paragraph_count = Paragraph.find(:all).length
    xhr :post, :create, {:paragraph => NEW_PARAGRAPH}
    paragraph, successful = check_attrs(%w(paragraph successful))
    assert successful, "Should be successful"
    assert_response :success
    assert_template 'create.rjs'
    assert_equal paragraph_count + 1, Paragraph.find(:all).length, "Expected an additional Paragraph"
  end

  def test_update
  	paragraph_count = Paragraph.find(:all).length
    post :update, {:id => @first.id, :paragraph => @first.attributes.merge(NEW_PARAGRAPH)}
    paragraph, successful = check_attrs(%w(paragraph successful))
    assert successful, "Should be successful"
    paragraph.reload
   	NEW_PARAGRAPH.each do |attr_name|
      assert_equal NEW_PARAGRAPH[attr_name], paragraph.attributes[attr_name], "@paragraph.#{attr_name.to_s} incorrect"
    end
    assert_equal paragraph_count, Paragraph.find(:all).length, "Number of Paragraphs should be the same"
    assert_response :redirect
    assert_redirected_to REDIRECT_TO_MAIN
  end

  def test_update_xhr
  	paragraph_count = Paragraph.find(:all).length
    xhr :post, :update, {:id => @first.id, :paragraph => @first.attributes.merge(NEW_PARAGRAPH)}
    paragraph, successful = check_attrs(%w(paragraph successful))
    assert successful, "Should be successful"
    paragraph.reload
   	NEW_PARAGRAPH.each do |attr_name|
      assert_equal NEW_PARAGRAPH[attr_name], paragraph.attributes[attr_name], "@paragraph.#{attr_name.to_s} incorrect"
    end
    assert_equal paragraph_count, Paragraph.find(:all).length, "Number of Paragraphs should be the same"
    assert_response :success
    assert_template 'update.rjs'
  end

  def test_destroy
  	paragraph_count = Paragraph.find(:all).length
    post :destroy, {:id => @first.id}
    assert_response :redirect
    assert_equal paragraph_count - 1, Paragraph.find(:all).length, "Number of Paragraphs should be one less"
    assert_redirected_to REDIRECT_TO_MAIN
  end

  def test_destroy_xhr
  	paragraph_count = Paragraph.find(:all).length
    xhr :post, :destroy, {:id => @first.id}
    assert_response :success
    assert_equal paragraph_count - 1, Paragraph.find(:all).length, "Number of Paragraphs should be one less"
    assert_template 'destroy.rjs'
  end

protected
	# Could be put in a Helper library and included at top of test class
  def check_attrs(attr_list)
    attrs = []
    attr_list.each do |attr_sym|
      attr = assigns(attr_sym.to_sym)
      assert_not_nil attr,       "Attribute @#{attr_sym} should not be nil"
      assert !attr.new_record?,  "Should have saved the @#{attr_sym} obj" if attr.class == ActiveRecord
      attrs << attr
    end
    attrs.length > 1 ? attrs : attrs[0]
  end
end
