require File.dirname(__FILE__) + '/../test_helper'

class ParagraphTest < Test::Unit::TestCase
  fixtures :paragraphs

	NEW_PARAGRAPH = {}	# e.g. {:name => 'Test Paragraph', :description => 'Dummy'}
	REQ_ATTR_NAMES 			 = %w( ) # name of fields that must be present, e.g. %(name description)
	DUPLICATE_ATTR_NAMES = %w( ) # name of fields that cannot be a duplicate, e.g. %(name description)

  def setup
    # Retrieve fixtures via their name
    # @first = paragraphs(:first)
  end

  def test_raw_validation
    paragraph = Paragraph.new
    if REQ_ATTR_NAMES.blank?
      assert paragraph.valid?, "Paragraph should be valid without initialisation parameters"
    else
      # If Paragraph has validation, then use the following:
      assert !paragraph.valid?, "Paragraph should not be valid without initialisation parameters"
      REQ_ATTR_NAMES.each {|attr_name| assert paragraph.errors.invalid?(attr_name.to_sym), "Should be an error message for :#{attr_name}"}
    end
  end

	def test_new
    paragraph = Paragraph.new(NEW_PARAGRAPH)
    assert paragraph.valid?, "Paragraph should be valid"
   	NEW_PARAGRAPH.each do |attr_name|
      assert_equal NEW_PARAGRAPH[attr_name], paragraph.attributes[attr_name], "Paragraph.@#{attr_name.to_s} incorrect"
    end
 	end

	def test_validates_presence_of
   	REQ_ATTR_NAMES.each do |attr_name|
			tmp_paragraph = NEW_PARAGRAPH.clone
			tmp_paragraph.delete attr_name.to_sym
			paragraph = Paragraph.new(tmp_paragraph)
			assert !paragraph.valid?, "Paragraph should be invalid, as @#{attr_name} is invalid"
    	assert paragraph.errors.invalid?(attr_name.to_sym), "Should be an error message for :#{attr_name}"
    end
 	end

	def test_duplicate
    current_paragraph = Paragraph.find_first
   	DUPLICATE_ATTR_NAMES.each do |attr_name|
   		paragraph = Paragraph.new(NEW_PARAGRAPH.merge(attr_name.to_sym => current_paragraph[attr_name]))
			assert !paragraph.valid?, "Paragraph should be invalid, as @#{attr_name} is a duplicate"
    	assert paragraph.errors.invalid?(attr_name.to_sym), "Should be an error message for :#{attr_name}"
		end
	end
end

